from sklearn.base import BaseEstimator
from sklearn.ensemble import RandomForestRegressor
from sklearn.linear_model import OrthogonalMatchingPursuit


class OmpForestRegressor(BaseEstimator):

    def __init__(self, forest_size, extracted_forest_size, seed=None):
        self._regressor = RandomForestRegressor(n_estimators=forest_size,
            random_state=seed)
        self._extracted_forest_size = extracted_forest_size

    def fit(self, X_train, y_train):
        self._forest = self._train_forest(X_train, y_train)

        self._weights = self._extract_subforest(X_train, y_train)

        return self

    @property
    def forest(self):
        return self._forest

    @property
    def weights(self):
        return self._weights

    def _train_forest(self, X_train, y_train):
        self._regressor.fit(X_train, y_train)
        forest = self._regressor.estimators_
        return forest
    
    def _extract_subforest(self, X_train, y_train):
        D = [[tree.predict([elem])[0] for tree in forest] for elem in X_train]
        omp = OrthogonalMatchingPursuit(n_nonzero_coefs=self._extracted_forest_size,
            fit_intercept=False, normalize=False)
        omp.fit(D, y_train)
        weights = omp.coef_
        return weights
